package com.fetchLocal.handler;

import android.util.Log;

import com.facebook.react.bridge.ReadableMap;
import com.fetchLocal.functional.BiFunction;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.HttpURLConnection;

public class PutPostBodyMethodHandler implements BiFunction<HttpURLConnection, ReadableMap, Void>
{
    private final BiFunction<HttpURLConnection, ReadableMap, String> contentTypeTransformer;

    private String contentType;

    public PutPostBodyMethodHandler(String contentType)
    {
        this.contentType = contentType;
        this.contentTypeTransformer = getContentTypeTransformer(contentType);
    }
    @Override
    public Void apply(HttpURLConnection connection, ReadableMap data) throws Exception
    {
        String body = this.contentTypeTransformer.apply(connection, data);
        setBody(connection, body);
        return null;
    }
    protected void setBody(HttpURLConnection connection, String body) throws IOException
    {
        connection.setDoOutput(true);
        DataOutputStream outputStream = new DataOutputStream(connection.getOutputStream());
        outputStream.writeBytes(body);
        outputStream.flush();
        outputStream.close();
    }
    protected BiFunction<HttpURLConnection, ReadableMap, String> getContentTypeTransformer(String contentType)
    {
        //System.out.println(contentType+"   here");
        switch (contentType) {
            case "application/json":
                return new JsonBodyHandler();
            default:
                String errMsg = "Content type not supported : " + contentType;
                Log.d(HttpMethodHandlerFactory.class.getName(), errMsg);
                throw new IllegalArgumentException(errMsg);
        }
    }
}
