package com.fetchLocal;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.JavaOnlyMap;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReadableMap;
import com.fetchLocal.handler.HttpMethodHandlerFactory;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.powermock.api.mockito.PowerMockito;
import org.powermock.core.classloader.annotations.PrepareForTest;
import org.powermock.modules.junit4.PowerMockRunner;

import needle.BackgroundThreadExecutor;
import needle.Needle;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyInt;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.validateMockitoUsage;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

@RunWith(PowerMockRunner.class)
@PrepareForTest({HttpMethodHandlerFactory.class, Arguments.class, Needle.class, FetchLocalModule.class})
public class FetchLocalModuleTest {

    @Before
    public void setUp() throws Exception {
        MockitoAnnotations.initMocks(this);
    }

    @After
    public void validate() {
        validateMockitoUsage();
    }

    @Test
    public void fetchLocalTest() throws Exception {
        Promise promise = mock(Promise.class);
        BackgroundThreadExecutor backgroundThreadExecutor = mock(BackgroundThreadExecutor.class);
        when(backgroundThreadExecutor.withThreadPoolSize(anyInt())).thenReturn(backgroundThreadExecutor);

        PowerMockito.mockStatic(Needle.class);
        PowerMockito.when(Needle.onBackgroundThread()).thenReturn(backgroundThreadExecutor);

        String link = "http://192.168.0.104:3000/home";
        ReactApplicationContext reactApplicationContext = mock(ReactApplicationContext.class);
        FetchLocalModule fetchLocalModule = new FetchLocalModule(reactApplicationContext);
        ReadableMap data = new JavaOnlyMap();
        fetchLocalModule.fetchLocal(link, data, promise);

        verify(backgroundThreadExecutor).execute(any(FetchLocalRunnable.class));
    }

    @Test
    public void fetchLocalTestException() throws Exception {
        Promise promise = mock(Promise.class);
        BackgroundThreadExecutor backgroundThreadExecutor = mock(BackgroundThreadExecutor.class);
        when(backgroundThreadExecutor.withThreadPoolSize(anyInt())).thenThrow(new RuntimeException("Test"));

        PowerMockito.mockStatic(Needle.class);
        PowerMockito.when(Needle.onBackgroundThread()).thenReturn(backgroundThreadExecutor);

        String link = "http://192.168.0.104:3000/home";
        ReactApplicationContext reactApplicationContext = mock(ReactApplicationContext.class);
        FetchLocalModule fetchLocalModule = new FetchLocalModule(reactApplicationContext);
        ReadableMap data = new JavaOnlyMap();
        try {
            fetchLocalModule.fetchLocal(link, data, promise);
            Assert.fail();
        } catch(Exception e) {
            Assert.assertEquals("Test", e.getMessage());
        }
    }
}