package com.fetchLocal.handler;

import com.facebook.react.bridge.JavaOnlyMap;
import com.facebook.react.bridge.ReadableMap;
import com.google.common.collect.Lists;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.util.List;

import static org.junit.Assert.*;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.spy;
import static org.mockito.Mockito.verify;

public class PutPostBodyMethodHandlerTest {
    private String contentType;
    private HttpURLConnection connection;
    private PutPostBodyMethodHandler putPostBodyMethodHandler;
    private JsonBodyHandler jsonBodyHandler;
    private String body;
    private OutputStream outputStream;
    private ReadableMap data;

    @Before
    public void setUp() throws Exception {
        contentType="application/json";
        putPostBodyMethodHandler= new PutPostBodyMethodHandler(contentType);
        jsonBodyHandler = new JsonBodyHandler();
        connection= Mockito.mock(HttpURLConnection.class);
        body= "{\"id\": \"appo_id\"}";
        outputStream=new ByteArrayOutputStream();
        List<Object> objectList= Lists.newArrayList();
        objectList.add("body");
        objectList.add(body);
        data = JavaOnlyMap.of(objectList.toArray());
    }

    @After
    public void tearDown() throws Exception {
    }

    @Test
    public void applyTest() throws Exception {
        PutPostBodyMethodHandler putPostBodyMethodHandler1 = spy(putPostBodyMethodHandler);
        doNothing().when(putPostBodyMethodHandler1).setBody(eq(connection),anyString());
        assertEquals(null,putPostBodyMethodHandler1.apply(connection,data));
        verify(putPostBodyMethodHandler1).setBody(eq(connection),anyString());
    }

    @Test
    public void setBodyTest() throws IOException {
        Mockito.when(connection.getOutputStream()).thenReturn(this.outputStream);
        putPostBodyMethodHandler.setBody(connection,body);
        verify(this.connection).setDoOutput(true);

    }

    @Test
    public void getContentTypeTransformerTest() {
        assertTrue(jsonBodyHandler.getClass().equals(putPostBodyMethodHandler.getContentTypeTransformer(contentType).getClass()));
        this.contentType="thk";
        Throwable exception= assertThrows(IllegalArgumentException.class, ()->putPostBodyMethodHandler.getContentTypeTransformer(contentType));
        assertEquals("Content type not supported : " + this.contentType,exception.getMessage());

    }
}